@@ -24,6 +24,7 @@ use crossterm::{
   terminal::{self, EnterAlternateScreen, LeaveAlternateScreen},
   terminal::{Clear, ClearType},
 };
+use smallvec::{smallvec, SmallVec};
 
 use ast_grep_config::RuleCollection;
 use ast_grep_core::Pattern;
@@ -116,25 +117,25 @@ fn collect_file_stats(
   Ok(())
 }
 
-pub fn filter_file_interactive(
+pub fn filter_file_rule(
   path: &Path,
   configs: &RuleCollection<SgLang>,
   trace: &ScanTrace,
-) -> Result<Vec<(PathBuf, AstGrep)>> {
+) -> Result<SmallVec<[AstGrep; 1]>> {
   let Some(lang) = SgLang::from_path(path) else {
-    return Ok(vec![]);
+    return Ok(smallvec![]);
   };
   let file_content = read_file(path)?;
   let grep = lang.ast_grep(file_content);
   collect_file_stats(path, lang, configs, trace)?;
-  let mut ret = vec![(path.to_path_buf(), grep.clone())];
+  let mut ret = smallvec![grep.clone()];
   if let Some(injected) = lang.injectable_sg_langs() {
     let docs = grep.inner.get_injections(|s| SgLang::from_str(s).ok());
     let inj = injected.filter_map(|l| {
       let doc = docs.iter().find(|d| *d.lang() == l)?;
       let grep = AstGrep { inner: doc.clone() };
       collect_file_stats(path, l, configs, trace).ok()?;
-      Some((path.to_path_buf(), grep))
+      Some(grep)
     });
     ret.extend(inj)
   }
@@ -143,40 +144,36 @@ pub fn filter_file_interactive(
 
 // sub_matchers are the injected languages
 // e.g. js/css in html
-pub fn filter_file_pattern(
+pub fn filter_file_pattern<'a>(
   path: &Path,
   lang: SgLang,
-  root_matcher: Option<Pattern<SgLang>>,
-  sub_matchers: impl Iterator<Item = (SgLang, Pattern<SgLang>)>,
-) -> Result<Vec<MatchUnit<Pattern<SgLang>>>> {
+  root_matcher: Option<&'a Pattern<SgLang>>,
+  sub_matchers: &'a [(SgLang, Pattern<SgLang>)],
+) -> Result<SmallVec<[MatchUnit<&'a Pattern<SgLang>>; 1]>> {
   let file_content = read_file(path)?;
   let grep = lang.ast_grep(&file_content);
-  let do_match = |ast_grep: AstGrep, matcher: Pattern<SgLang>| {
+  let do_match = |ast_grep: AstGrep, matcher: &'a Pattern<SgLang>| {
     let fixed = matcher.fixed_string();
     if !fixed.is_empty() && !file_content.contains(&*fixed) {
       return None;
     }
-    let has_match = ast_grep.root().find(&matcher).is_some();
-    has_match.then(|| MatchUnit {
+    Some(MatchUnit {
       grep: ast_grep,
       path: path.to_path_buf(),
       matcher,
     })
   };
-  let mut ret = vec![];
+  let mut ret = smallvec![];
   if let Some(matcher) = root_matcher {
     ret.extend(do_match(grep.clone(), matcher));
   }
   let injections = grep.inner.get_injections(|s| SgLang::from_str(s).ok());
-  for (i_lang, matcher) in sub_matchers {
-    let Some(injection) = injections.iter().find(|i| *i.lang() == i_lang) else {
-      continue;
-    };
-    let injected = AstGrep {
-      inner: injection.clone(),
-    };
-    ret.extend(do_match(injected, matcher));
-  }
+  let sub_units = injections.into_iter().filter_map(|inner| {
+    let (_, matcher) = sub_matchers.iter().find(|i| *inner.lang() == i.0)?;
+    let injected = AstGrep { inner };
+    do_match(injected, matcher)
+  });
+  ret.extend(sub_units);
   Ok(ret)
 }
 
