@@ -9,9 +9,9 @@ use thiserror::Error;
 
 use std::borrow::Cow;
 use std::collections::{HashMap, HashSet};
-use std::sync::{Arc, RwLock, RwLockReadGuard, RwLockWriteGuard, Weak};
+use std::sync::{Arc, Weak};
 
-pub struct Registration<R>(Arc<RwLock<HashMap<String, R>>>);
+pub struct Registration<R>(Arc<HashMap<String, R>>);
 
 impl<R> Clone for Registration<R> {
   fn clone(&self) -> Self {
@@ -20,19 +20,18 @@ impl<R> Clone for Registration<R> {
 }
 
 impl<R> Registration<R> {
-  // TODO: this is sooo wrong
-  pub(crate) fn read(&self) -> RwLockReadGuard<HashMap<String, R>> {
-    self.0.read().unwrap()
-  }
-  fn write(&self) -> RwLockWriteGuard<HashMap<String, R>> {
-    self.0.write().unwrap()
+  #[allow(clippy::mut_from_ref)]
+  fn write(&self) -> &mut HashMap<String, R> {
+    // SAFETY: `write` will only be called during initialization and
+    // it only insert new item to the hashmap. It is safe to cast the raw ptr.
+    unsafe { &mut *(Arc::as_ptr(&self.0) as *mut HashMap<String, R>) }
   }
 }
 pub type GlobalRules<L> = Registration<RuleCore<L>>;
 
 impl<L: Language> GlobalRules<L> {
   pub fn insert(&self, id: &str, rule: RuleCore<L>) -> Result<(), ReferentRuleError> {
-    let mut map = self.write();
+    let map = self.write();
     if map.contains_key(id) {
       return Err(ReferentRuleError::DuplicateRule(id.into()));
     }
@@ -41,7 +40,7 @@ impl<L: Language> GlobalRules<L> {
     // TODO: we can skip check here because insertion order
     // is guaranteed in deserialize_env
     if rule.check_cyclic(id) {
-      return Err(ReferentRuleError::CyclicRule);
+      return Err(ReferentRuleError::CyclicRule(id.to_string()));
     }
     Ok(())
   }
@@ -65,16 +64,8 @@ pub struct RuleRegistration<L: Language> {
 
 // these are shit code
 impl<L: Language> RuleRegistration<L> {
-  fn get_local(&self) -> RwLockReadGuard<HashMap<String, Rule<L>>> {
-    self.local.read()
-  }
-
-  fn get_global(&self) -> RwLockReadGuard<HashMap<String, RuleCore<L>>> {
-    self.global.read()
-  }
-
-  pub fn get_rewriters(&self) -> GlobalRules<L> {
-    self.rewriters.clone()
+  pub fn get_rewriters(&self) -> &HashMap<String, RuleCore<L>> {
+    &self.rewriters.0
   }
 
   pub fn from_globals(global: &GlobalRules<L>) -> Self {
@@ -85,22 +76,14 @@ impl<L: Language> RuleRegistration<L> {
     }
   }
 
-  pub fn with_rewriters(&self, rewriters: &GlobalRules<L>) -> Self {
-    Self {
-      local: self.local.clone(),
-      global: self.global.clone(),
-      rewriters: rewriters.clone(),
-    }
-  }
-
-  pub fn get_ref(&self) -> RegistrationRef<L> {
+  fn get_ref(&self) -> RegistrationRef<L> {
     let local = Arc::downgrade(&self.local.0);
     let global = Arc::downgrade(&self.global.0);
     RegistrationRef { local, global }
   }
 
-  pub fn insert_local(&self, id: &str, rule: Rule<L>) -> Result<(), ReferentRuleError> {
-    let mut map = self.local.write();
+  pub(crate) fn insert_local(&self, id: &str, rule: Rule<L>) -> Result<(), ReferentRuleError> {
+    let map = self.local.write();
     if map.contains_key(id) {
       return Err(ReferentRuleError::DuplicateRule(id.into()));
     }
@@ -109,17 +92,19 @@ impl<L: Language> RuleRegistration<L> {
     // TODO: we can skip check here because insertion order
     // is guaranteed in deserialize_env
     if rule.check_cyclic(id) {
-      return Err(ReferentRuleError::CyclicRule);
+      return Err(ReferentRuleError::CyclicRule(id.to_string()));
     }
     Ok(())
   }
 
-  pub(crate) fn get_local_util_vars<'a>(&'a self) -> HashSet<&'a str> {
+  pub(crate) fn insert_rewriter(&self, id: &str, rewriter: RuleCore<L>) {
+    self.rewriters.insert(id, rewriter).expect("should work");
+  }
+
+  pub(crate) fn get_local_util_vars(&self) -> HashSet<&str> {
     let mut ret = HashSet::new();
-    let utils = self.get_local();
+    let utils = &self.local.0;
     for rule in utils.values() {
-      // SAFETY: self will retain the reg_ref and guarantee &Rule is valid
-      let rule = unsafe { &*(rule as *const Rule<L>) as &'a Rule<L> };
       for v in rule.defined_vars() {
         ret.insert(v);
       }
@@ -137,31 +122,35 @@ impl<L: Language> Default for RuleRegistration<L> {
   }
 }
 
-pub struct RegistrationRef<L: Language> {
-  local: Weak<RwLock<HashMap<String, Rule<L>>>>,
-  global: Weak<RwLock<HashMap<String, RuleCore<L>>>>,
+/// RegistrationRef must use Weak pointer to avoid
+/// cyclic reference in RuleRegistration
+struct RegistrationRef<L: Language> {
+  local: Weak<HashMap<String, Rule<L>>>,
+  global: Weak<HashMap<String, RuleCore<L>>>,
 }
-// these are shit code
 impl<L: Language> RegistrationRef<L> {
-  pub fn unref(&self) -> RuleRegistration<L> {
-    let local = Registration(self.local.upgrade().unwrap());
-    let global = Registration(self.global.upgrade().unwrap());
-    RuleRegistration {
-      local,
-      global,
-      rewriters: Default::default(),
-    }
+  fn get_local(&self) -> Arc<HashMap<String, Rule<L>>> {
+    self
+      .local
+      .upgrade()
+      .expect("Rule Registration must be kept alive")
+  }
+  fn get_global(&self) -> Arc<HashMap<String, RuleCore<L>>> {
+    self
+      .global
+      .upgrade()
+      .expect("Rule Registration must be kept alive")
   }
 }
 
 #[derive(Debug, Error)]
 pub enum ReferentRuleError {
-  #[error("Rule `{0}` is not found.")]
-  RuleNotFound(String),
+  #[error("Rule `{0}` is not defined.")]
+  UndefinedUtil(String),
   #[error("Duplicate rule id `{0}` is found.")]
   DuplicateRule(String),
-  #[error("Rule has a cyclic dependency in its `matches` sub-rule.")]
-  CyclicRule,
+  #[error("Rule `{0}` has a cyclic dependency in its `matches` sub-rule.")]
+  CyclicRule(String),
 }
 
 pub struct ReferentRule<L: Language> {
@@ -184,8 +173,7 @@ impl<L: Language> ReferentRule<L> {
   where
     F: FnOnce(&Rule<L>) -> T,
   {
-    let registration = self.reg_ref.unref();
-    let rules = registration.get_local();
+    let rules = self.reg_ref.get_local();
     let rule = rules.get(&self.rule_id)?;
     Some(func(rule))
   }
@@ -194,11 +182,22 @@ impl<L: Language> ReferentRule<L> {
   where
     F: FnOnce(&RuleCore<L>) -> T,
   {
-    let registration = self.reg_ref.unref();
-    let rules = registration.get_global();
+    let rules = self.reg_ref.get_global();
     let rule = rules.get(&self.rule_id)?;
     Some(func(rule))
   }
+
+  pub(super) fn verify_util(&self) -> Result<(), ReferentRuleError> {
+    let rules = self.reg_ref.get_local();
+    if rules.contains_key(&self.rule_id) {
+      return Ok(());
+    }
+    let rules = self.reg_ref.get_global();
+    if rules.contains_key(&self.rule_id) {
+      return Ok(());
+    }
+    Err(ReferentRuleError::UndefinedUtil(self.rule_id.clone()))
+  }
 }
 
 impl<L: Language> Matcher<L> for ReferentRule<L> {
@@ -244,7 +243,7 @@ mod test {
     let rule = ReferentRule::try_new("test".into(), &registration)?;
     let rule = Rule::Matches(rule);
     let error = registration.insert_local("test", rule);
-    assert!(matches!(error, Err(ReferentRuleError::CyclicRule)));
+    assert!(matches!(error, Err(ReferentRuleError::CyclicRule(_))));
     Ok(())
   }
 
@@ -254,7 +253,17 @@ mod test {
     let rule = ReferentRule::try_new("test".into(), &registration)?;
     let rule = Rule::All(o::All::new(std::iter::once(Rule::Matches(rule))));
     let error = registration.insert_local("test", rule);
-    assert!(matches!(error, Err(ReferentRuleError::CyclicRule)));
+    assert!(matches!(error, Err(ReferentRuleError::CyclicRule(_))));
+    Ok(())
+  }
+
+  #[test]
+  fn test_cyclic_not() -> Result {
+    let registration = RuleRegistration::<TS>::default();
+    let rule = ReferentRule::try_new("test".into(), &registration)?;
+    let rule = Rule::Not(Box::new(o::Not::new(Rule::Matches(rule))));
+    let error = registration.insert_local("test", rule);
+    assert!(matches!(error, Err(ReferentRuleError::CyclicRule(_))));
     Ok(())
   }
 
