@@ -1,8 +1,4 @@
-use crate::source::Content;
-use std::borrow::Cow;
-use std::cmp::Ordering;
-use std::ops::Range;
-
+#![warn(clippy::doc_overindented_list_items)]
 /**
   This module is for indentation-sensitive replacement.
 
@@ -115,6 +111,10 @@ use std::ops::Range;
   The steps 3,4 and steps 5,6 are similar. We can define a `replace_with_indent` to it.
   Following the same path, we can define a `extract_with_deindent` for steps 1,2
 */
+use crate::source::Content;
+use std::borrow::Cow;
+use std::cmp::Ordering;
+use std::ops::Range;
 
 /// We assume NEW_LINE, TAB, SPACE is only one code unit.
 /// This is sufficiently true for utf8, utf16 and char.
@@ -204,7 +204,7 @@ where
   let new_line = get_new_line::<C>();
   for line in lines {
     ret.push(new_line.clone());
-    ret.extend(leading.iter().cloned());
+    ret.extend(leading.clone());
     ret.extend(line.iter().cloned());
   }
   ret
