@@ -21,6 +21,8 @@ fn test_cpp_pattern() {
     "template <typename T> typename std::enable_if<std::is_signed<T>::value, T>::type;",
   );
   test_match("if ($A)", "if (a | b) abc;");
+  // see https://github.com/ast-grep/ast-grep/issues/1791
+  test_match("struct $A: $B", "struct A: B {}");
 }
 
 fn test_replace(src: &str, pattern: &str, replacer: &str) -> Result<String, TSParseError> {
