@@ -3,6 +3,7 @@ import test from 'ava'
 import {
   registerDynamicLanguage,
   parse,
+  findInFiles,
 } from '../index'
 
 const { platform, arch } = process
@@ -13,18 +14,18 @@ const canTestDynamicLang = isAppleSilicon || isX64Linux
 
 if (isAppleSilicon) {
   registerDynamicLanguage({
-    myjson: {
+    json: {
       libraryPath: "../../benches/fixtures/json-mac.so",
       languageSymbol: "tree_sitter_json",
-      extensions: ["myjson"],
+      extensions: ["json"],
     }
   })
 } else if (isX64Linux) {
   registerDynamicLanguage({
-    myjson: {
+    json: {
       libraryPath: "../../benches/fixtures/json-linux.so",
       languageSymbol: "tree_sitter_json",
-      extensions: ["myjson"],
+      extensions: ["json"],
     }
   })
 }
@@ -34,11 +35,32 @@ test('test load custom lang', t => {
     t.pass('This test is not available on this platform')
     return
   }
-  const sg = parse('myjson', '{"test": 123}')
+  const sg = parse('json', '{"test": 123}')
   const root = sg.root()
   const node = root.find("123")!
   t.truthy(node)
   t.is(node.kind(), 'number')
   const no = root.find("456")
   t.falsy(no)
+})
+
+test('discover file', async t => {
+  if (!canTestDynamicLang) {
+    t.pass('This test is not available on this platform')
+    return
+  }
+  await findInFiles('json', {
+    paths: ['../'],
+    matcher: {
+      rule: {
+        kind: 'string'
+      }
+    }
+  }, (error, nodes) => {
+    t.falsy(error)
+    t.truthy(nodes)
+    t.is(nodes[0].kind(), 'string')
+    const file = nodes[0].getRoot().filename()
+    t.assert(file.endsWith('.json'))
+  })
 })
\ No newline at end of file
